function [R, x0_w,x0_lip,x0_lac,x0_min,x0_cas,x0_whey,x0_whey_de,x0_NPN]=UF_filter(m_r, ph, pr, pl,x0_cas_end)
% By: Tomas Tungen Kjellstrm 
% Start date: 5.02.2017
% NTNU
% Details:
% This function generates the permeate and retentate (main stream) stream
% flow and composition of an ultra filtration membrane:
% m_r is the massflow (kg/min) of raw milk.
% ph is the inlet pressure, pr is the retentate pressure and pl is the permeate pressure (Kpa).
% theta is the feed to permeate mass flow ratio.
 

% Export details:
% R is the mass stream of condensed milk, kg/h.
% x0_w, x0_lip, x0_lac, x0_min and x0_cas, x0_whey, x0_whey_de and x0_NPN
% are the mass fraction of water, fat, lactose, minerals, casein, 
% unnaturated whey, denaturated whey and non protein nitrogen of the condensed milk

% Refrences:
% [1] Hagenes, K. (2010). Produksjon av meieriprodukter. First edition.
% Oslo: Baneforlaget

% [2] Datta, N., & Tomasula, P. M. (2015). 
% Emerging dairy processing technologies.
% Retrieved from http://ebookcentral.proquest.com

if nargin==0
    m_r=100;     % mass flow of raw milk, kg/h
    ph=5;        % feed pressure, kPa
    pr=4;        % retentate pressure, kPa
    pl=1;        % permeate pressure, kPa
    x0_cas_end=4;   % retentate (main stream) casein mass fraction, w/w %
end
% Importing the feed flow in the membrane:
[m_fs,x_fs_pro,x_fs_lip,x_fs_lac,x_fs_min,x_fs_w]= sep_mass_balance(m_r);

% Rename the feed into membrane standard nomeclature:
L_f=m_fs;
xf_w=x_fs_w;
xf_lip=x_fs_lip;
xf_lac=x_fs_lac;
xf_min=x_fs_min;
xf_pro=x_fs_pro;

% Insert an statement that the desired end compostion of the retentate must
% be greater or equal to the feed composition:
if x0_cas_end/100<xf_pro*0.8
fprintf('The entered casein mass fraction of the retentate is less than the mass fraction of casein in the fat regulated milk from the separator\n')
error('The casein content of the retentate milk must be greater or equal to the fat regulated milk from the separator')
end
% The transmembrane pressure (TMP): average pressure gradient
TMP=(ph+pr)/2-pl;

% From a table in [1], p 62: w/w
% Feed after a separator:
c_fw=0.9416; % water
c_flip=0.0005; % fat
c_flac=0.0431; % lactose
c_fmin=0.0057; % minerals
c_fpro=0.0079; %protein
% Protein composition:
c_fcas=0.0023; % casein
c_fwhey=0.0017; % naturated whey
c_fNPN=0.0022; % Nitrogen non-protein
c_fwhey_de=c_fpro-c_fcas-c_fwhey-c_fNPN;% denaturated whey

% Retentate:
c_rw=0.8317;
c_rlip=0.009;
c_rlac=0.0522;
c_rmin=0.0078;
c_rpro=0.1017;
% Protein composition:
c_rcas=0.0495; % casein
c_rwhey=0.0226; % naturated whey
c_rNPN=0.0069; % Nitrogen non-protein
c_rwhey_de=c_rpro-c_rcas-c_rwhey-c_rNPN;% denaturated whey

% permeate
c_pw=0.9563;
c_plip=0.0; 
c_plac=0.0377;
c_pmin=0.0052;
c_ppro=0.0018;
% Protein composition:
c_pcas=0.0001; % casein
c_pwhey=0.00; % naturated whey
c_pNPN=0.0016; % Nitrogen non-protein
c_pwhey_de=c_ppro-c_pcas-c_pwhey-c_pNPN;% denaturated whey

% Teoretical water flux: [2]
J_w_test=500/3600; % kg water/m^2/s (density = 1 kg/l)
TMP_test=50; % kPa
mu=1.04e-6; % kPa s
L=16e-6; % m
% Kozeny-Carman relation defines the permeate flow as:
% L_f=k*A*(TMP)//L => J=L_f/A
% Reevalute into flux: J=k*TMP//L and solves for K:
K_w=J_w_test*mu/TMP_test/L; % kg/m^2

% Calculating the remaining permeability constants relative to the water
% permeability:
L_f_test=10; %kg tot/s
V_p_test=L_f_test*(c_fw-c_rw)/(c_pw-c_rw); % kg tot/s
V_p_w_test=V_p_test*c_pw; % kg water/s
A_test=V_p_w_test/(J_w_test);% m^2


% Component fluxes: kg/m^2/s
J_lac_test=V_p_test*c_plac/A_test;
J_lip_test=V_p_test*c_plip/A_test;
J_min_test=V_p_test*c_pmin/A_test;
J_cas_test=V_p_test*c_pcas/A_test;
J_whey_test=V_p_test*c_pwhey/A_test;
J_whey_de_test=V_p_test*c_pwhey_de/A_test;
J_NPN_test=V_p_test*c_pNPN/A_test;

% Permeability constants: K_i[=] kg/m^2
K_lac=J_lac_test*mu/TMP_test/L;
K_lip=J_lip_test*mu/TMP_test/L;
K_min=J_min_test*mu/TMP_test/L;
K_cas=J_cas_test*mu/TMP_test/L;
K_whey=J_whey_test*mu/TMP_test/L;
K_whey_de=J_whey_de_test*mu/TMP_test/L;
K_NPN=J_NPN_test*mu/TMP_test/L;

% Actual permeate fluxes
J_w=K_w*TMP/mu;
J_lac=K_lac*TMP/mu;
J_lip=K_lip*TMP/mu;
J_min=K_min*TMP/mu;
J_cas=K_cas*TMP/mu;
J_whey=K_whey*TMP/mu;
J_whey_de=K_whey_de*TMP/mu;
J_NPN=K_NPN*TMP/mu;

% Permeate composition is equal the sum of all fluxes:
J_tot=J_w+J_lac+J_lip+J_min+J_cas+J_whey+J_whey_de+J_NPN;
C_p_w=J_w/J_tot;
C_p_lac=J_lac/J_tot;
C_p_lip=J_lip/J_tot;
C_p_min=J_min/J_tot;
C_p_cas=J_cas/J_tot;
C_p_whey=J_whey/J_tot;
C_p_whey_de=J_whey_de/J_tot;
C_p_NPN=J_NPN/J_tot;
C_p_tot=C_p_w+C_p_lac+C_p_lip+C_p_min+C_p_cas+C_p_whey+C_p_whey_de+C_p_NPN;

% Forloop session to calculate the composition of the retentate and the
% mass flows of the permeate and the retentate:
maxiter=1e+8;
tol=1e-7;
theta=0.4;
k=1;
for k=1:maxiter
% Retentate composition:
% Inserting the theta to the mass balance, followed up by solving with
% L0: retentate stream, V_p: permeate stream, L_f: feed stream
% respect ot C_L0_i: theta=V_p/L_f, L_f*c_f_i=V_p*c_p_i+L0*c_L0_i

x0_w=xf_w/(1-theta)-theta/(1-theta)*C_p_w;
x0_lac=xf_lac/(1-theta)-theta/(1-theta)*C_p_lac;
x0_lip=xf_lip/(1-theta)-theta/(1-theta)*C_p_lip;
x0_min=xf_min/(1-theta)-theta/(1-theta)*C_p_min;
x0_cas=xf_pro*0.8/(1-theta)-theta/(1-theta)*C_p_cas; % 80 of milk protein is casein
% whey and NPN content
cf_pro_noncas=xf_pro*0.2;
c_f_pro_noncas=c_fwhey+c_fwhey_de+c_fNPN;
x0_whey=cf_pro_noncas*(c_fwhey/c_f_pro_noncas)/(1-theta)-theta/(1-theta)*C_p_whey;
x0_whey_de=cf_pro_noncas*(c_fwhey_de/c_f_pro_noncas)/(1-theta)-theta/(1-theta)*C_p_whey_de;
x0_NPN=cf_pro_noncas*(c_fNPN/c_f_pro_noncas)/(1-theta)-theta/(1-theta)*C_p_NPN;

% Mass balance test:
x0_tot=x0_w+x0_lac+x0_lip+x0_min+x0_cas+x0_whey+x0_whey_de+x0_NPN;

% Termination loop
diff=x0_cas-x0_cas_end/100; % -1>= diff<=1
if abs(diff)<=tol |k>=maxiter % The desired composition or the maximum number og iterations is reached
    if k>=maxiter
        tol('maximum number of iterations reached in the UF filter,the desired value is not obtained');
    end
    break
end
% water deficit
if diff>=tol
theta_new=theta(end)-diff;
elseif diff<-tol % water surplus
theta_new= theta(end)+abs(diff);
end
% Updating the new theta valu and iteration number
theta=theta_new;
k=k+1;
end
% test values: No technical function, only for verification
k_filter=k;
x0_cas;
x0_tot;
x0_min;
theta;
if x0_cas<0 || x0_min<0|| x0_lac<0|| x0_whey<0 || x0_whey_de<0|| x0_NPN<0 ||x0_w<0
fprintf('The mass balance of the ultra filtration membrane is not fullfilled.\n')
error('One of the retentate compositions is negative')
end

% Permeate and retentate flow:
V_p=L_f*theta;
R=L_f-V_p;

% mass balance test
lactose_test=L_f*xf_lac/(C_p_lac*V_p+x0_lac*R);
mineral_test=L_f*xf_min/(C_p_min*V_p+x0_min*R);

% Neede membrane area:
A=V_p*C_p_w/J_w;
end