function [V_tot, S_coag, m_w_tot, m_persipitate, m_coa_add, P_coag, B_coag, pH_coag,rho_milk_0]=coagulation(m_r, ph, pr, pl,x0_cas_end)
% By: Tomas Tungen Kjellstrm 
% Start date: 20.02.2017
% NTNU
% Details:
% This function imports values from the fermentation stage and further
% inoculate an amount of rennet to start forming a coagel of the casein
% protein in the fermented milk.

% Export details:
% V_tot is the batch volume, l.
% S_coag is the lactose evolution troughout the coagulation, g/l.
% m_w_tot, m_persipitate, m_coa_add are the total mass of water (m_w_tot),
% persipitated casein and CaCl2 in the vat (m_persipitate)
% and the mass added to the vat during the coagulation phase (m_coa_add).
% p_coag is the molecular evolution of lactate during the coagulation,
% mol/l.
% B_coag is the biomass concentration evolution of the coagulation, g/l.
% pH_coag is the pH evolution of the coagulation.
% rho_milk_0 is the average density of the vat after addition of rennet and CaCl2, kg/l

% Refrences:
% [1] 
% Bittante et al. (2013). 
% Prolonged observation and modelling of milk coagulation, 
% curd firming, and syneresis. International Dairy Journal. Vol. 29(2)

% [2]
% Bylund, G. (2003). Dairy processing handbook.
% Second edition. Malm, Sweden: Tetra Pak Processing Systems AB

% [3]
% yaas, J (1995). P. hD:
% STUDY OF DIFFUSION PROCESSES AND LACTIC ACID PRODUCTION BY 
% LACTOBACILLUS HELVETICUS IMMOBILIZED IN Ca-ALGINATE GEL.
% PhD thesis. University of Trondheim

% [4]
% Bouguettoucha, A., Balannec, B., Amrane, A.(2011)
% Unstructured Models for Lactic Acid Fermentation - A Review. Volume 49(1) 
% Food Technology and Biotechnology 

% [5]
% Aylward, G., Findlay, T. (2008). SI Chemical Data. 
% Sixth edition. Milton, Australia: John Wiley & Sons Australia Ltd 

% assumptions:
% It is assumed that the added calsium cloride has a high
% affinity to the casein and these two components make-up the coagel.

if nargin==0
    m_r=100; % amount of raw milk (kg)
    ph=5;    % feed pressure (kPa)
    pr=4;    % retentate pressure (kPa)
    pl=1;    % permeate pressure (kPa)
    x0_cas_end=4;   % retentate (main stream) casein mass fraction, w/w %
end

% Importing data from fermenter function:
[t_ferm, pH, S, C_cas_ferm,m_w,m_lip,m_casein,V,B,m_ferm,P,Kp]= fermenter(m_r, ph, pr, pl,x0_cas_end);

% Importing data on the initial mineral consentration:
[R, x0_w,x0_lip,x0_lac,x0_min,x0_cas,x0_whey,x0_whey_de,x0_NPN]=UF_filter(m_r, ph, pr, pl,x0_cas_end);
% Addition  of calcium cloride solution [2]:
C_Ca_Cl2=35; % Salinity, g/kg
m_cal=10/100*m_r; % 5-20 g/100 kg (g CaCl2)
rho_saltwater=1.025; % kg/l
v_cal=m_cal/C_Ca_Cl2/rho_saltwater; % l saltwater added
m_saltwater=m_cal/C_Ca_Cl2; % kg saltwater added
m_water_salt=m_saltwater-m_cal/1000;% mass of pure water in the saltwater (kg)

% Addition of pure rennet [1]:
r=200/0.255; % 200 l/10 ml raw milk => 200 l/0.255 g casein
v_rennet_pure=(r*1e-6)*(m_casein*1e+3); % l rennet
c_rennet=0.8;
v_rennet=v_rennet_pure/c_rennet; % diluted rennet (1.6 w %)
v_water_rennet=v_rennet*(1-c_rennet);
m_water_rennet=v_water_rennet; % assuming density equal 1 kg/l
m_rennet=v_rennet; % assuming density equal 1 kg/l

% Total volume content in the vat: l
V_tot=V+v_rennet+v_cal;
% Total mass entering the system during the coagulation (kg):
m_coa_add=m_saltwater+m_rennet;
% Total mass in the vat: kg
m_tot_coag=m_ferm+m_coa_add;
% Total mass of water in the vat: kg
m_w_tot=m_w+m_water_salt+m_water_rennet;

% Initial concentration: kg/kg total
c0_w=m_w/m_tot_coag;
c0_lip=R*x0_lip/m_tot_coag;
c0_lac=(S(end)*1e-3*V)/m_tot_coag; % (g/l*1e-3 kg/g*l)/kg tot
c0_min=R*x0_min/m_tot_coag;
c0_cas=R*x0_cas/m_tot_coag;
c0_whey=R*(x0_whey+x0_whey_de+x0_NPN)/m_tot_coag;
c0_lactate=P(end)*1e-3*V/m_tot_coag;
c0_bio=B(end)*1e-3*V/m_tot_coag;

F_0=c0_lip*100;
SNF_0=100-F_0-c0_w*100;
rho_milk_0=100/(F_0/0.93+SNF_0/1.608+100*c0_w); % kg/l

C0_cas=C_cas_ferm*V/V_tot; % g/l

% Slow coagulating model [1]:
% Time from addition of rennet to coagulation start:
RCT=24.8; % Rennet coagulation time (min)
% Asymptotical potential curd firmness (theoretical maximum):
CF_P=64.1;

% Curd-firming and curd syneresis rate constant 
k_CF=0.074;% % 100^-1 min ^-1 (fraction/ min)
k_SF=0.0160;
% Maximum curd firmness (mm) and time:
CF_max=35.7; 
t_CFmax=50.2;
% Length intervall:
Sl=1e+4;
% Time vector before and after RCT:
t_before=linspace(0,RCT,Sl);
t_after=linspace(RCT,100,Sl);
% The sensoric responce is neighter relevant or possible to measure at low
% curd fimness.
% The curd firmness is therfore defined equal zero before RCT:
CF_before=zeros(1,Sl);
% Curd firmness accordingly to the four-parameter model of Bittante et al.
% (2013) [1]:
CF_after=CF_P.*(1-exp(-k_CF.*(t_after-RCT))).*exp(-k_SF.*(t_after-RCT));
% Curd-firmness and time vector:
CF=[CF_before CF_after]; % mm
t_coag=[t_before t_after];    % min

% Desired fimness:
% Must be less than the obtainable maximum
CF_des=CF_max*0.8;

% Coagulation time solver
x=RCT+4; % Initial time
tol=1e-5; % tolerance
maxiter=1e+8; % maximum number of iterations
k=1; % iteration 1

for k=1:maxiter
    CF_c=CF_P*(1-exp(-k_CF*(x-RCT)))*exp(-k_SF*(x-RCT))-CF_des;
% Stops the iteration at time x if the desired fimness (CF_des) is reached
    if abs(CF_c)<=tol || k>maxiter % Termination
        if k>=maxiter
           error('maximum number of iterations reached\n')
        end
        break
    end
    if CF_c<=-tol % Curd firmess deflicit
    x=x+abs(CF_c)/10;
    elseif CF_c>tol   % Curd firmess surplus
    x=x-abs(CF_c)/10;
    end
    k=k+1;
end
iterations=k;
deviation=CF_c;
% Coagulation time and end firmness: min
coag_time=x;
CF_end=CF_P.*(1-exp(-k_CF.*(coag_time-RCT))).*exp(-k_SF.*(coag_time-RCT));

% Inital fermentation conditions of the coagulation: g/l
S_0= (c0_lac*1e+3)*rho_milk_0;
B_0=(c0_bio*1e+3)*rho_milk_0;
P_0=(c0_lactate*1e+3)*rho_milk_0;

% Fermentation during the coagulation:
% The initial conditions of the coagulation are the end conditions of the
% fermentation: [3]
Y_ps=0.83; % lactic acid yield
Y_bs=1-Y_ps; % biomass yield, g biomass formed/ g lactose consumed
Y_pb= Y_ps/Y_bs; % product over biomass yield
n=0.3; % must be the same as in fermenter
mu_max=0.94*(S_0/3.48)^n; % h^-1
Ks=0.22; % g/l
% The fermentation is more inhibited during the coagulation, than during
% the acidification due to the pH.
Kpc=Kp/1e+0; % Kp is defined in fermenter 


% Buffering capacity of milk:
pka_buffer=5.5;
% Using the Henderson-Hasselbach equation to determine the needed molar
% ratio of base and acid: A_HA_ratio=[A^-]/[HA]
A_HA_ratio=10^(pH(end)-pka_buffer);
% Concentration of buffer: casein used as a measure to regulate [7]
alfa_s1=38/97*C0_cas*rho_milk_0*1e+3;% g/l
Mw_s1=(22+23.7)/2*1e+3; % g/mol (Da)
c_s1=alfa_s1/Mw_s1; % mol/l
alfa_s2=10/97*C0_cas*rho_milk_0*1e+3;
Mw_s2=25e+3;
c_s2=alfa_s2/Mw_s2;
kappa=36/97*C0_cas*rho_milk_0*1e+3;
Mw_kappa=19e+3;
c_kappa=kappa/Mw_kappa;
beta=13/97*C0_cas*rho_milk_0*1e+3;
Mw_beta=24e+3;
c_beta=beta/Mw_beta;
citric=0.2/100*1e+3*rho_milk_0; % g/l
Mw_citric=192.124;
c_citric=citric/Mw_citric;

c_buff=8*(c_s1+c_s2+c_kappa+c_beta+c_citric); % mol/l
HA_mol_0=c_buff*1/(1+A_HA_ratio);
A_mol_0=c_buff-HA_mol_0;

Ka=10^(-3.86); % acid constant of lactic acid (C3H6O3) [4]
Mw_lactic_acid= 90.08; % g/mol

% Monod equation is solved for varying rates in a for loop 
tol2=0.01;
kf=1;
maxiter=1e+5;
for kf=1:maxiter
% Reaction rate between kt and kt+1: [4]
mu(kf)=mu_max.*S_0(end)./(S_0(end)+Ks).*(Kpc./(10^(-pH(end))+Kpc(end)));   
% Time step size: 
t=0.01/60; % h
% Biomass concentration vector, using the integrated Monod equation:
B_new(kf)=B_0(end).*exp(mu(end).*t);
dB=B_new-B_0(end); % Change (dB>=0)

% Lactic acid concentration vector:
dP=Y_pb.*dB(end); % dP>=0
P_new(kf)=P_0(end)+dP(end);
P_new_mol=P_new(end)./Mw_lactic_acid;

% Buffer concentrations:
A_mol=A_mol_0-P_new_mol;
HA_mol=HA_mol_0+P_new_mol;

% New pH:
pH(kf)=pka_buffer+log10(A_mol(end)/HA_mol(end));

% Lactose:
dS=-dB(end)-dP(end); % dS<=0
S_new(kf)=S_0(end)+dS(end);
% Time elapsed at iteration kt:
t_total=t*kf(end)*60; % min
t_diff=t_total-coag_time;
% Reenstating the consentrations as the new inital state for the for loop:
B_0=B_new;
P_0=P_new;
S_0=S_new;
pH_end=pH;

% Creating a termination loop, that stops the iterations at the 
% calculated coagulation time above:
if kf>=maxiter
    fprintf('Maximum number of iterations in the coaglation fermentation is reached')
%elseif t_diff<0
 %   fprintf('The used fermentation time in the coagulation is greater than the actual coagulation time')
elseif abs(t_diff)<=tol2
    break
end
kf=kf+1;
end
% Information matrices:
B_coag=B_0;
P_coag=P_0;
S_coag=S_0;
pH_coag=pH_end;

% End and comparison results of the iteration process:
B_coag_end=B_coag(end);
P_coag_end=P_coag(end);
S_coag_end=S_coag(end);
pH_coag_end=pH_coag(end)

% The coagulation time and the needed fermentation time should be identical
coag_time;
t_ferm_coag=t_total;
 
% Coagel:
m_persipitate=m_casein+m_cal/1000; % mass percipitate of the coagel

% Illustrations
figure(4)
plot(t_coag,CF)
hold on
plot(coag_time,CF_end,'o')
hold off
xlabel('Time (min)'); ylabel('Curd firmness (mm)')
title('Curd-firmness over time model')
legend('Slow-coagulating')
end
