function [m_curd,m_whey_in_int,c_coagel_con_int,c_water_con_int,c_lac_con_int,c_lactate_con_int,c_bio_con_int,c_whey_con_int,c_min_con_int,c_CaCl2_con_int,c_lip_con_int]=draining_continuous(m_r,ph,pr,pl,x0_cas_end,n,R2)
% By: Tomas Tungen Kjellstrm 
% Start date: 10.04.2017
% NTNU
% Details:
% This function makes a model for a continous draining stage of curd from
% the whey, by decantering of the cheese.

% Export details:
% m_curd is the total mass left in the vat at time t (time intervall),
% m_whey_in_int is the mass of whey in the vat (time intervall),
% c_coagel_con_int, c_water_con_int, c_lac_con_int, c_lactate_con_int,
% c_bio_con_int and c_whey_con_int are the mass fraction time intervall 
% of the components left in curd: the coagel, water, lactose, lactate,
% biomass and whey (protein)

% Refrences:
% [1]
% Bylund, G. (2003). Dairy processing handbook.
% Second edition. Malm, Sweden: Tetra Pak Processing Systems AB

% [2]
% Daviau, C. (2000). Rennet coagulation of skim milk and curd drainage:
% Effect of pH, casein concentration, ionic strength 
% and heat treatment Vol 80. page
% 397-415. Le Lait 

% [3]
% Floury, J et al. (2009). Reducing salt level in food: Part 2.
% Modelling salt diffusion in model cheese systems with regards to 
% their composition. Volume 4. Pages 1621-1628. Food Science and Technology
% Column information: [1]
% Single column capacity : 1000-2500 kg/h
% curd:whey ratio 1:3.5-5.0

if nargin==0
    m_r=100;     % mass flow of raw milk (kPa)
    ph=5;        % feed pressure of the UF-membrane (kPa)
    pr=4;        % retentate pressure (kPa)
    pl=1;        % permeate pressure (kPa)
    x0_cas_end=4;   % retentate (main stream) casein mass fraction, w/w %
    n=0.3;       % volume removal ratio of the first drainage
    R2=3.5;      % whey to curd mass ratio after second drainage
end
% Importing data on the inital conditions of the curd as a starting point
% of the drainage, assuming the vat to work as the buffering tank:
[m_vat_int,m_whey_int,c_coa_int,c_lac_int,c_lactate_int,c_w_int,c_bio_int,c_whey_int,c_min_int,c_lip_int,c_CaCl2,rho_liq]=second_drain(m_r,ph,pr,pl,x0_cas_end,n,R2);

% Column pressure:
P_col=2; % atm
P_atm=1;
% Inital fractions in the vat:
C_w_0=c_w_int(end);
C_lac_0=c_lac_int(end);
C_lactate_0=c_lactate_int(end);
C_bio_0=c_bio_int(end);
C_wheypro_0=c_whey_int(end);
C_min_0=c_min_int(end);
C_CaCl2_0=c_CaCl2(end);
C_coagel_0=c_coa_int(end);
C_lip_0=c_lip_int(end);

% Inital component masses in the vat:
m_w_0=C_w_0*m_vat_int(end);
m_coa_0=C_coagel_0*m_vat_int(end);
m_lac_0=C_lac_0*m_vat_int(end);
m_lactate_0=C_lactate_0*m_vat_int(end);
m_bio_0=C_bio_0*m_vat_int(end);
m_wheypro_0=C_wheypro_0*m_vat_int(end);
m_min_0=C_min_0*m_vat_int(end);
m_CaCl2_0=C_CaCl2_0*m_vat_int(end);
m_lip_0=C_lip_0*m_vat_int(end);
% Initial mass whey (solution) in the vat:
m_whey_0=m_whey_int(end);% kg whey
% Assuming an constant mass ratio of components in the whey, kg i/kg whey:
R_w=m_w_0/m_whey_0;
R_lac=m_lac_0/m_whey_0;
R_lactate=m_lactate_0/m_whey_0;
R_bio=m_bio_0/m_whey_0;
R_whey=m_wheypro_0/m_whey_0;
R_min=m_min_0/m_whey_0;
R_CaCl2=m_CaCl2_0/m_whey_0;
% Dresired water mass fraction after drainage in the curd (total
% composition): [3]
c_w_end=1-0.44; % Usual dry-matter content of brining cheese is 37-44 w%


% Drainage kinetics [2]: Mesophilic, CN=-1, IS=-1
W_1=893/2*0.95; % g/kg expelled (95 % expelled at time t_1)
t_1=53;    % min
W_2=827/2*0.95;
t_2=10.25;

error=1e-5;
maxiter=1e+7;
k=1;
% initial guess
t=1; % min
for k=1:maxiter
% Pressure ratio:(F_col/F_atm)^n=(P_col/P_atm)^n=m_col/m_atm: FJERNES????
n=1;
R_p=(P_col/P_atm)^n;

% per thousandth of whey removed from the curd:
W=R_p.*(W_1*(1-exp(-t(end)/t_1))+W_2*(1-exp(-t(end)/t_2))); % g/kg
% Total amount of whey removed from the vat at time t: kg
d_whey=W(end)*1e-3*m_vat_int(end);

% water concentration at time t:
% Cw(t)= m_w(t)/m_tot(t)=(m_whey(t)*R_w)/(m_coa_0+m_whey(t))
% where: m_whey(t)=m_whey_0-d_whey(t)
C_w=(m_whey_0)*R_w/(m_vat_int(end)-d_whey(end))-d_whey*R_w/(m_vat_int(end)-d_whey(end));

% Difference between the obtained and the desired water consentration:
difference=C_w-c_w_end;

diff=abs(difference);

% converging unit of the loop system
if k>=maxiter
    fprintf('maximum number of iterations reached')
    break
elseif diff<=error
    break
end
if difference>error % water surplus in the vat
    t_test=t+diff/100;
elseif difference<-error % water deficit in the vat
    t_test=t-diff/100;
end
t=t_test;
k=k+1;
end
% Draining time and time intervall:
S=5e+1;% intervall length
time_drain=t(end); % draining time, min
time_drain_int=linspace(0,time_drain,S);
% draining intervall:
% g whey/kg total
W_int=R_p.*(W_1.*(1-exp(-time_drain_int./t_1))+W_2.*(1-exp(-time_drain_int./t_2))); 
W_int(end)
% Mass of whey in the curd intervall: 
m_whey_in_int=m_whey_0-(W_int.*m_vat_int(end)./1000); % kg
% Total mass of the curd during the drainage:
m_curd=m_coa_0+m_lip_0+m_whey_in_int; % kg
% Mass fractions in the curd during the draining column: kg/kg total
c_coagel_con_int=m_coa_0./m_curd;
c_lip_con_int=m_lip_0./m_curd;
c_water_con_int=R_w.*m_whey_in_int./m_curd; % kg w/kg whey*kg whey/kg tot
c_water_con_int(end);
c_lac_con_int=R_lac.*m_whey_in_int./m_curd;
c_lactate_con_int=R_lactate.*m_whey_in_int./m_curd;
c_bio_con_int=R_bio.*m_whey_in_int./m_curd;
c_whey_con_int=R_whey.*m_whey_in_int./m_curd;
c_min_con_int=R_min.*m_whey_in_int./m_curd;
c_CaCl2_con_int=R_CaCl2.*m_whey_in_int./m_curd;
% Mass balance check: C_tot_int=1
c_check=c_coagel_con_int+c_lip_con_int+c_water_con_int+c_lac_con_int+c_lactate_con_int+c_bio_con_int+c_whey_con_int+c_min_con_int+c_CaCl2_con_int;
end