function [t_ferm, pH, S, C_cas_ferm,m_w,m_lip,m_casein,V,B,m_ferm,P,Kp]= fermenter(m_r, ph, pr, pl,x0_cas_end)
% By: Tomas Tungen Kjellstrm 
% Start date: 13.03.2017
% NTNU
% Details:
% This function generates the needed fermentation time and 
% lactose concentration to yield the optimum pH for protein coagulation of 
% of the milk

% Exporting details:
% t is the time in minutes for the fermentation to reach the desired level
% of acidity.
% pH is the pH evolution from adition of starter cultures (prepared to be
% in logphase) to the end of the fermentation.
% Svv is the lactose evolution.
% x_cas_ferm is the casein mass compositon in the fermented milk.
% m_w, m_lip and m_casein are he mass of water, fat and casein at the end
% of the fermentation.
% V is the batch volume, l.
% B is the biomass evolution vector of the fermented milk (g/l).
% m_ferm is the total mass in the vat after addition of the starter
% culture (kg)
% P is the lactic acid evolution vector of the fermented milk [g/l]

% Refrences
% [1]
% yaas, J (1995). P. hD:
% STUDY OF DIFFUSION PROCESSES AND LACTIC ACID PRODUCTION BY 
% LACTOBACILLUS HELVETICUS IMMOBILIZED IN Ca-ALGINATE GEL.
% PhD thesis. University of Trondheim

% [2]
% Doran, Pauline M. (2012). Bioprocess Engineering Principles.
% Second edition. Academic Press

% [3]
% Aylward, G., Findlay, T. (2008). SI Chemical Data. 
% Sixth edition. Milton, Australia: John Wiley & Sons Australia Ltd 

% [4] 
% Hui (2007). Handbook of Food Products Manufacturing. Volume 2. John
% Wiley & Sons

% [5]
% Bylund, G. (2003). Dairy processing handbook.
% Second edition. Malm, Sweden: Tetra Pak Processing Systems AB

% [6]
% Bouguettoucha, A., Balannec, B., Amrane, A.(2011)
% Unstructured Models for Lactic Acid Fermentation - A Review. Volume 49(1) 
% Food Technology and Biotechnology 

% [7]
% Product information sheet:Casein from bovine milk. Sigma Aldrich.
% downloaded 15.06.2017 available from: 
% http://www.sigmaaldrich.com/content/dam/sigma-aldrich/docs/Sigma/Product_Information_Sheet/c6554pis.pdf

% [8]
% Whittier, E.O. (1929). Buffercapacity of milk and milk constituents. 
% vol 83. p:79-88. Journal of Biological Chemistry

% assumptions:
% it takes one hour to pretreat the milk, meaning one hour basis of the
% continious production part over to the batch vise production. etc 100
% kg/h feed means that the vat is filled with the fat, water and
% lactose regulated milk.

if nargin==0
    m_r=100; % amount of raw milk (kg)
    ph=5;    % feed pressure (kPa)
    pr=4;    % retentate pressure (kPa)
    pl=1;    % permeate pressure (kPa)
   x0_cas_end=4;   % retentate (main stream) casein mass fraction, w/w %
end
% Assuming lactic acid is the only acidifying component
Mw_lactic_acid= 90.08; % g/mol

% Importing data on the input data:
[R, x0_w,x0_lip,x0_lac,x0_min,x0_cas,x0_whey,x0_whey_de,x0_NPN]=UF_filter(m_r, ph, pr, pl,x0_cas_end);

% Start culture:
m_cult=2.0/100*R; % kg start culture (1-2 W% of the regulated vat milk)
B_CFU=17e+11; % Cells/l
% Mass fraction biomass: assuming a colony weight of 1 pg
B_0_cult=B_CFU*1e-12*1e-3; % kg biomass/kg starterculture
W_cult=1-B_0_cult;

% Mass and initial mass fraction of the components: kg i/kg total
m_ferm = R+m_cult;
C0_w =(W_cult*m_cult+R*x0_w)/m_ferm;
C0_lip = R*x0_lip/m_ferm;
C0_lac = R*x0_lac/m_ferm;
C0_min = R*x0_min/m_ferm;
C0_cas = R*x0_cas/m_ferm;
C0_whey = R*x0_whey/m_ferm;
C0_whey_de = R*x0_whey_de/m_ferm;
C0_NPN = R*x0_NPN/m_ferm;
C0_bio = B_0_cult*m_cult/m_ferm;

% Average density of the pretreated milk, assuming the same density at the 
% start of fermentation as at the end due to no evaporation or gas 
% production and that the lactose only change into an other Solid-Non-Fat [5]: kg/l
rho_milk_0=100/(100*C0_lip/0.93+100*(C0_lac+C0_min+C0_cas+C0_whey+C0_whey_de+C0_NPN+C0_bio)/1.608+100*C0_w);

% Volume of the milk: l
V=m_ferm/(rho_milk_0);
% initial mass/volume concentration of lactose and biomass: g/l
S_0=C0_lac*rho_milk_0*1000;
B_0=C0_bio*rho_milk_0*1000; % g biomass/l

% Micaelis-Menten kinetics
% Lactobacillus Helveticus CNRZ 303: [1]
% assuming the Ks and Km are identical
Km=0.22; % g/l
% optimal pH 7 and 42 Celsius with lactose as the limiting substrate and
% that the doubling time increases linearly with the availability of
% substrate:
n=0.1;
mu_max=0.94*(S_0/3.48)^n; % h^-1 
% pH 5.9
mu_low =0.86; 

% Lactic acid and biomass yield of lactose:
Y_ps=0.83; % g lactic acid formed/ g lactose consumed
Y_bs=1-Y_ps; % theoretical maximum is 0.8 g cells formed/g lactose consumed [2]

% lactic acid over biomass yield:
Y_pb=Y_ps/Y_bs;

% Inhibition constant:
% assuming non-competitive inhibition with the inhibition constant Kp: [6]
Kp=Km/1e+5;
P_0=0; % no initial lactic acid in the milk
% ideal pH for adition of rennet for enzymatic coagulation of Gouda type
% cheeses is 6.6:
pideal=6.6; 

% Buffering capacity of milk: [8]
pka_buffer=5.5;
pH=6.7;
% Using the Henderson-Hasselbach equation to determine the needed molar
% ratio of base and acid: A_HA_ratio=[A^-]/[HA]
A_HA_ratio=10^(pH-pka_buffer);
% Concentration of buffer: casein and citric acid used as a measure to regulate [7]
alfa_s1=38/97*C0_cas*rho_milk_0*1e+3;% g/l
Mw_s1=(22+23.7)/2*1e+3; % g/mol (Da)
c_s1=alfa_s1/Mw_s1; % mol/l
alfa_s2=10/97*C0_cas*rho_milk_0*1e+3;
Mw_s2=25e+3;
c_s2=alfa_s2/Mw_s2;
kappa=36/97*C0_cas*rho_milk_0*1e+3;
Mw_kappa=19e+3;
c_kappa=kappa/Mw_kappa;
beta=13/97*C0_cas*rho_milk_0*1e+3;
Mw_beta=24e+3;
c_beta=beta/Mw_beta;
citric=0.2/100*1e+3*rho_milk_0; % g/l
Mw_citric=192.124;
c_citric=citric/Mw_citric;
c_buff=8*(c_s1+c_s2+c_kappa+c_beta+c_citric); % mol/l
% Calculating the concentrations of the buffer acid and base:
HA_mol_0=c_buff*1/(1+A_HA_ratio);
A_mol_0=c_buff-HA_mol_0;

err=0.001;
max=1e+7;
kt=1;
for kt=1:max
% Reaction rate: [6]
mu(kt)=mu_max.*S_0(end)./(S_0(end)+Km).*Kp./(10^(-pH(end))+Kp);
% step size: time
t=0.01/60; % [h] 
% Integrate and differenciate the Monod equation: d[B]/dt=*[B]
B_new(kt)=B_0(end).*exp(mu(end).*t); % biomass
dB=B_new(end)-B_0(end);

% Lactic acid formation:
dP=Y_pb.*dB(end);
P_new(kt)=P_0(end)+dP(end);
P_new_mol(kt)=P_new(end)./Mw_lactic_acid;

% Buffer concentrations:
A_mol=A_mol_0-P_new_mol(end);
HA_mol=HA_mol_0+P_new_mol(end);

% New pH:
pH(kt)=pka_buffer+log10(A_mol(end)/HA_mol(end));

% lactose:
dS=-dB(end)-dP(end);
S_new(kt)=S_0(end)+dS(end);
% Time elapsed:
t_total=t*kt(end)*60;

% Reenstating the consentrations as the new inital state for the for loop:
B_0=B_new;
P_0=P_new;
P_0_mol=P_new_mol;
S_0=S_new;
pH_end=pH;

% Establishing a termination circuit to stop iteration when the ideal pH is
% reached:
diff=pH(end)-pideal;
if kt>=max
    error('Maximum number of iterations in the fermentation is reached and the value is incorrrect')
elseif abs(diff)<=err
    break
elseif diff<-err
    fprintf('The accurate pH is not obtained at this configuration, due to the for loop')
    break
end
% Updating the iteration number
kt=kt+1;
end


% Doubling time: initial and end
double_0=log(2)/mu(1)*60 
double_end=log(2)/mu(end)*60
% End values of the fermentation:
B_ferm_end=B_0(end);
P_ferm_end=P_0(end);
P_ferm_mol_end=P_0(end)/Mw_lactic_acid;
S_ferm_end=S_0(end);
t_ferm=t_total;
pH_ferm_end= pH(end);



% Updating the composition vectors: g/l
B=B_0;
S=S_0;
P=P_0;
P_mol=P_0_mol;
C_cas_ferm=C0_cas*rho_milk_0*1000;

% Amount casein after fermentation, assuming casein is not fermented (kg):
m_casein=R*x0_cas;

% Amount of water in the vat after fermentation, assuming no water
% production or evaporation(kg):
m_w=m_ferm*C0_w;

% Amout of fat in the vat after fermentation, assuming fat is not fermented
% (kg):
m_lip=R*x0_lip;
t_span=linspace(0,t_ferm,kt);

% pH-evolution
figure(1)
plot(t_span,pH)
xlabel('time, min')
ylabel('pH')
title('pH over time')

figure(2)
plot(pH(2:end),(abs(P_mol(1:end-1)-P_mol(2:end))./(abs(pH(1:end-1)-pH(2:end)))))
xlabel('pH of the milk')
ylabel('Buffer intensity, d[P]/dpH')
title('Buffer intensity over fermentation')

figure(3)
plot(t_span,P)
title('Lactic acid concentration during the fermentation')
xlabel('Time [min]')
ylabel('Lactic acid concentration [g/l]')
end