function [V_inter,m_vat,C_whey,C_lac,C_lactate,C_bio,C_min,C_w,m_coagel_solids,pH_1_d_end,B_1d_end,P_1d_end,S_1d_end]=first_drain(m_r,ph,pr,pl,x0_cas_end,n)
% By: Tomas Tungen Kjellstrm 
% Start date: 31.03.2017
% NTNU
% Details:
% This function makes a model for the first drainage of the coagel has been
% cut into the desired size.

% Exporting details:
% V_inter is the volume of the batch from start to end of the first drain, l
% m_vat is the mass present in the vat from start to end of the first drain, kg
% C_whey, C_lac, C_lactate, C_bio and C_w are the mass fraction of
% whey(protein), lactose, lactate, biomass and water in the whey solution.
% m_coagel_solids is the mass of /persipitated part of the coagel, kg.
% pH_1_d_end, B_1_d_end, P_1_d_end, S_1_d_end are the values of pH, biomass, lactate and
% lactose (g/l) after the first drain.

% Assumptions:
% The drain contains: water, whey protein, lactose, lactic acid and minerals, where the
% coagel conserves all fat and casein.

% Refrences:
% [1] 
% Bylund, G. (2003). Dairy processing handbook.
% Second edition. Malm, Sweden: Tetra Pak Processing Systems AB

% [2]
% Daviau, C. (2000). Rennet coagulation of skim milk and curd drainage:
% Effect of pH, casein concentration, ionic strength 
% and heat treatment Vol 80. page
% 397-415. Le Lait 

% [3]
% Whittier, E.O. (1929). Buffercapacity of milk and milk constituents. 
% vol 83. p:79-88. Journal of Biological Chemistry
if nargin==0
    m_r=100;     % mass flow of raw milk (kg)
    ph=5;        % feed pressure of UF-membrane (kPa)
    pr=4;        % retentate pressure (kPa)
    pl=1;        % permeate pressure (kPa)
    x0_cas_end=4;   % retentate (main stream) casein mass fraction, w/w %
    n=0.3;       % volume removal ratio of the first drain (30 %)
end

% Importing data on the coagulation:
[V_tot, S_coag, m_w_tot, m_persipitate, m_coa_add, P_coag, B_coag, pH_coag,rho_milk_0]=coagulation(m_r, ph, pr, pl,x0_cas_end);

% Importing data on the mass initially entering the vat:
[R, x0_w,x0_lip,x0_lac,x0_min,x0_cas,x0_whey,x0_whey_de,x0_NPN]=UF_filter(m_r, ph, pr, pl,x0_cas_end);
% importing data on the mass initially in the vat:
%[t_ferm, pH, S, C_cas_ferm,m_w,m_lip,m_casein,V,B,m_ferm,P,Kp]= fermenter(m_r, ph, pr, pl,x0_cas_end)
[t, pH, S, x_cas_ferm,m_w,m_lip,m_casein,V,B,m_ferm,P,Kp]= fermenter(m_r, ph, pr, pl,x0_cas_end);
% initial total mass:
m_tot_0=m_ferm+m_coa_add;
% Cuting of the coagel:
% Based on values from the dairy at Jren
t_cut=linspace(0,11,12); % 10-12 min, visual determination
V_cut=ones(length(t_cut),1)'.*V_tot;
% Resting time after the cutting, to increase the durabillity of the cutted
% cubes: 
x0_whey_tot=(x0_whey+x0_whey_de+x0_NPN)*R/m_tot_0; % kg whey/ kg tot
t_rest=linspace(t_cut(end),5+t_cut(end),6); % 5 min
V_rest=ones(length(t_rest),1)'.*V_tot;

% Draining:
% Drainage kinetics [2]: Mesophilic, CN=-1, IS=-1
W_1=893/2*0.95; % g/kg expelled (95 % expelled at time t_1)
t_1=53;    % min
W_2=827/2*0.95;
t_2=10.25;

% inital whey concentration:
m_whey_0=m_tot_0-m_persipitate;
C_whey_0=(m_whey_0)/m_tot_0; % kg whey sol/kg tot
% whey solution solute concetration:
c_lac=S_coag(end)*V_tot/1000/m_whey_0;
c_lactate=(S_coag(1)-S_coag(end)-B_coag(end))*V_tot/1000/m_whey_0;
c_bio=B_coag(end)*V_tot/1000/m_whey_0;
c_whey=(x0_whey+x0_whey_de+x0_NPN)*R/m_whey_0;
c_w=1-c_lac-c_lactate-c_bio-c_whey;
% The whey is assumed to not contain casein, lipids or minerals:[3]
rho_whey=100/(100*((c_lac+c_lactate+c_bio+c_whey)/1.63)+100*c_w); % kg/l
V_whey_0=m_whey_0/rho_whey;

% Volume of the batch after the first drain: 
V_tot_end=V_tot*(1-n); % liter

% Using a forloop session to calculate the needed draining time:
error=1e-2;
maxiter=1e+5;
k=1;
t=15; % min
m_whey=m_whey_0;
for k=1:maxiter
% per thousandth of whey removed from the curd:
W=W_1*(1-exp(-t(end)/t_1))+W_2*(1-exp(-t(end)/t_2)); % g whey/kg tot
% mass and volume of whey left in the vat at time t:
m_whey_test=m_whey_0-W/1000*m_tot_0;
V_whey_test=m_whey_test/rho_whey;
% Total volume left in the vat at time t:
V_tot_test=V_tot-(V_whey_0-V_whey_test);
difference=V_tot_test(end)-V_tot_end;

% 
if difference>error
    t_test=t(end)+abs(difference)/10;
elseif difference<-error
    t_test=t(end)-abs(difference)/10;
end
% Stops the iteration:
if abs(difference)<=error | k>=maxiter
    break
end
t=t_test;
k=k+1;
end

% Time to complete the draining:
t_fd=t(end); % min (7-12 min at Jren)
% Draining intervall
S=1e+3;
t_drain=linspace(t_rest(end),t_rest(end)+t_fd,S);

% Calculating the volume evolution with the draining time:
% per thousandth of whey removed from the curd:
t_d_int=linspace(0,t_fd,S);
W=W_1.*(1-exp(-t_d_int./t_1))+W_2.*(1-exp(-t_d_int./t_2)); % g/kg
m_whey_int=m_whey_0-W./1000.*m_tot_0;
V_whey_int=m_whey_int./rho_whey;
V_diff=V_whey_0-V_whey_int;
V_tot_int=V_tot-V_diff;
% Testing if volume removal is equal n=(V_0-V)/V_0
n_test=(V_tot_int(1)-V_tot_int(end))/V_tot_int(1);

% Total first drain time vector:
time=[t_cut t_rest t_drain];

% Volume of the batch at time t:
V_inter=[V_cut V_rest V_tot_int]; % l

% Volume of the liquid leaving the vat, containing, water, solved whey,
% lactose, lacate, biomass and minerals:
V_liq=V_tot-V_inter; % l
% mass balance check:
mass_tot=m_ferm+m_coa_add; % total added to the vat (kg)
m_whey=R*x0_whey_tot; % mass of whey protein in the vat (kg)

m_lac= S_coag(end)/1000*V_tot; % mass of lacotse in the vat after coagulation (kg)
m_bio=B_coag(end)/1000*V_tot; % biomass in the vat after coagulation (kg)
m_lactate=P_coag(end)/1000*V_tot; % lactate in the vat after coagulation (kg)
m_min=x0_min*R;
m_w_tot;% mass of water in the vat (kg)
m_lip=R*x0_lip; % mass of fat in the vat (kg)
% Mass of the solids and fat in the coagel (kg)
m_coagel_solids=m_persipitate+m_lip; 

% Composition of the dispersed liquid (whey) after coagulation, mass fraction:
m_liq=m_whey+m_lac+m_bio+m_lactate+m_w_tot+m_min; % mass of the liquid (kg)
C_whey=m_whey/m_liq; % kg/kg_tot
C_lac=m_lac/m_liq;
C_bio=m_bio/m_liq;
C_lactate=m_lactate/m_liq;
C_w=m_w_tot/m_liq;
C_min=m_min/m_liq;
F=0;% assuming all fat is maintained on the surface of the casein micelles
SNF=(1-C_w)*100-F; % Solids Non-fat, %

% Density of the dispersed liquid [1]: kg/l
rho_liq=100/(F/0.93+SNF/1.608+C_w*100);

% mass drained off at time t:kg
m_drain=rho_liq.*V_liq;
% vat mass after draining at time t: kg
m_vat=mass_tot-m_drain;

% Fermentation:
% The fermentation is divided into to segments, before draining
% and draining:

% Fermentation before draining: constant volume
S_0=S_coag(end);
B_0=B_coag(end);
P_0=P_coag(end);
pH=pH_coag(end);
Y_ps=0.83;
Y_bs=1-Y_ps;
Y_pb=Y_ps/Y_bs;

m=0.3; % must be the same as in fermenter
mu_max=0.94*(S_0/3.48)^m; % h^-1
Ks=0.22; % g/l
% The fermentation is more inhibited during the coagulation, than during
% the acidification due to the pH.
Kpc=Kp/1e+0; % Kp is defined in fermenter

% Buffering capacity of milk:
pka_buffer=5.5;
% Using the Henderson-Hasselbach equation to determine the needed molar
% ratio of base and acid: A_HA_ratio=[A^-]/[HA]
A_HA_ratio=10^(pH(end)-pka_buffer);

C0_cas=m_casein/m_tot_0;
% Concentration of buffer: casein used as a measure to regulate [3]
alfa_s1=38/97*C0_cas*rho_milk_0*1e+3;% g/l
Mw_s1=(22+23.7)/2*1e+3; % g/mol (Da)
c_s1=alfa_s1/Mw_s1; % mol/l
alfa_s2=10/97*C0_cas*rho_milk_0*1e+3;
Mw_s2=25e+3;
c_s2=alfa_s2/Mw_s2;
kappa=36/97*C0_cas*rho_milk_0*1e+3;
Mw_kappa=19e+3;
c_kappa=kappa/Mw_kappa;
beta=13/97*C0_cas*rho_milk_0*1e+3;
Mw_beta=24e+3;
c_beta=beta/Mw_beta;
citric=0.2/100*1e+3*rho_milk_0; % g/l
Mw_citric=192.124;
c_citric=citric/Mw_citric;

c_buff=8*(c_s1+c_s2+c_kappa+c_beta+c_citric); % mol/l
HA_mol_0=c_buff*1/(1+A_HA_ratio);
A_mol_0=c_buff-HA_mol_0;

Ka=10^(-3.86); % acid constant of lactic acid (C3H6O3) 
Mw_lactic_acid= 90.08; % g/mol

% Monod equation is solved for varying rates in a for loop 
tol2=0.01;
kf=1;
maxiter=1e+5;
for kf=1:maxiter
% Reaction rate between kt and kt+1: 
mu(kf)=mu_max.*S_0(end)./(S_0(end)+Ks).*(Kpc./(10^(-pH(end))+Kpc(end)));   
% Time step size: 
t=0.01/60; % h
% Biomass concentration vector, using the integrated Monod equation:
B_new(kf)=B_0(end).*exp(mu(end).*t);
dB=B_new-B_0(end); % Change (dB>=0)

% Lactic acid concentration vector:
dP=Y_pb.*dB(end); % dP>=0
P_new(kf)=P_0(end)+dP(end);
P_new_mol=P_new(end)./Mw_lactic_acid;

% Buffer concentrations:
A_mol=A_mol_0-P_new_mol;
HA_mol=HA_mol_0+P_new_mol;

% New pH:
pH(kf)=pka_buffer+log10(A_mol(end)/HA_mol(end));

% Lactose:
dS=-dB(end)-dP(end); % dS<=0
S_new(kf)=S_0(end)+dS(end);
% Time elapsed at iteration kt:
t_total=t*kf(end)*60; % min
t_diff=t_total-t_rest(end);
% Reenstating the consentrations as the new inital state for the for loop:
B_0=B_new;
P_0=P_new;
S_0=S_new;
pH_end=pH;

% Creating a termination loop, that stops the iterations at the 
% calculated coagulation time above:
if kf>=maxiter
    fprintf('Maximum number of iterations in the first drain fermentation is reached')
elseif abs(t_diff)<=tol2
    break
end
kf=kf+1;
end
% Information matrices after the cutting and resting:
B_1_d_b=B_0;
P_1_d_b=P_0;
S_1_d_b=S_0;
pH_1_d_b=pH_end;

% Fermentaion during the draining:
% Assuming constant mass of buffer in the vat and that the fermentation
% occurs prior to the drain
S_0=S_1_d_b(end);
B_0=B_1_d_b(end);
P_0=P_1_d_b(end);
pH=pH_1_d_b(end);
A_HA_ratio_d=10^(pH-pka_buffer);
% Average buffering capcasity over the draining:
c_buff_d=c_buff*(1+1/(1-n))/2; % mol/l
HA_mol_0=c_buff_d*1/(1+A_HA_ratio_d);
A_mol_0=c_buff-HA_mol_0;
t_drain=linspace(t_rest(end),t_rest(end)+t_fd,S);

tol3=0.01;
maxiter=1e+6;
kd1=1;
for kd1=1:S-1
% Reaction rate between kd1 and kd1+1: 
mu(kd1)=mu_max.*S_0(end)./(S_0(end)+Ks).*(Kpc./(10^(-pH(end))+Kpc(end)));
% Time step size: 
t=(t_drain(2)-t_drain(1))/60;% h
% Biomass concentration vector, using the integrated Monod equation: g/l
B_new(kd1)=B_0(end).*exp(mu(end).*t);
B_new(end);
dB=B_new(end)-B_0(end); % Change (dB>=0)
% Lactic acid concentration vector:
dP=Y_pb.*dB(end); % dP>=0
P_new(kd1)=P_0(end)+dP(end);

P_new_mol=P_new(end)./Mw_lactic_acid;

% Buffer concentrations:
A_mol_inter=A_mol_0-P_new_mol;
HA_mol_inter=HA_mol_0+P_new_mol;

% New pH:
pH(kd1)=pka_buffer+log10(A_mol_inter(end)/HA_mol_inter(end));

% Lactose:
dS=-dB(end)-dP(end); % dS<=0
S_new(kd1)=S_0(end)+dS(end);

% Time elapsed at iteration kt:
t_total=t*kd1(end)*60; % min
% t_diff=t_total-t_rest(end);

% Concentration change as a result of vat volume:
m_whey_int(kd1); % Total mass of whey (solution) in the vat at iteration kd1 (kg)
V_tot_int(kd1); % Total volume of the vat (l)
% Lactose:
m_S=S_new(end)*V_tot_int(kd1);
C_S=m_S(end)./m_whey_int(kd1);
m_S_new=C_S(end).*m_whey_int(kd1+1);
S_0=m_S_new(end)./V_tot_int(kd1+1);

% Biomass:
m_B=B_new(end).*V_tot_int(kd1);
C_B=m_B(end)./m_whey_int(kd1);
m_B_new=C_B(end).*m_whey_int(kd1+1);
B_0=m_B_new(end)./V_tot_int(kd1+1);

% Lactate:
m_P=P_new(end).*V_tot_int(kd1);
C_P=m_P(end)./m_whey_int(kd1);
m_P_new=C_P(end).*m_whey_int(kd1+1);
P_0=m_P_new(end)./V_tot_int(kd1+1);

kd1=kd1+1;
end
pH_1_d_end=pH(end)
time_test=t_total;
B_1d_end=B_0(end);
P_1d_end=P_0(end);
S_1d_end=S_0(end);
% Illustrations
figure(5)
plot(time,V_inter)
hold on
plot(time,V_liq)
hold off
xlabel('time, min')
ylabel('Volume, l')
legend('batch volume','drained volume')
title('First drain')

end