function [pH_heat,c_lac,c_lactate,c_w,c_bio,c_whey,c_min,c_CaCl2,m_whey]= heating(m_r, ph, pr, pl,x0_cas_end,n)
% By: Tomas Tungen Kjellstrm 
% Start date: 03.04.2017
% NTNU
% Details:
% This function generates the heating of the curd, after the first
% drainage. The temperature is increased by adding steam/ water mixture to
% kill the lactic acid producing bacteria, to make theese bacteria start
% lycilling and emiting enzyme and to stop the acidification of the curd.

% Exporting details:
% pH_heat is the pH-evolution of the heating.
% c_lac, c_lactate, c_w, c_bio and c_whey, c_min, c_CaCl2 are the mass fraction evolution
% of lactose, lactate, water, biomass and whey protein, NaCl and CaCl2 in the whey 
% solution during the heating.
% m_whey is the mass of whey present in the vat during the heating.

% Refrences:
% [1]
% Bylund, G. (2003). Dairy processing handbook.
% Second edition. Malm, Sweden: Tetra Pak Processing Systems AB

% [2] Aylward, G., Findlay, T. (2008). SI Chemical Data. 
% Sixth edition. Milton, Australia: John Wiley & Sons Australia Ltd 

% [3] Y. H. Hui (2007). Handbook of Food Products Manufacturing. volume 2.
% John Wiley & Sons

% [4] Gomez et al (1995). The heat capacity of proteins. PROTEINS:
% Sturcture, Functions and Genetics 22:404-412. John Wiley & Sons

% assumptions:
% The main microbial activity of the lactic acid producing bacteria are
% assumed to stop at the start of this phase and it is therefore no
% fermentation in this phase, only dilution of the whey solution

if nargin==0
    m_r=100;     % mass flow of raw milk (kg)
    ph=5;        % Feed pressure of the UF-membrane (kPa)
    pr=4;        % retentate pressure (kPa)
    pl=1;        % permeate pressure (kPa)
    x0_cas_end=4;   % retentate (main stream) casein mass fraction, w/w %
    n=0.3;       % volume removal ratio of the first drain
end
% Importing data on the comoposition of the whey solution and 
% the amount of solid mass of the coagel after the first drain:
[V_int,m_vat,C_whey,C_lac,C_lact,C_bio,C_min,C_w,m_coagel,pH_1_d,B_1d,P_1d,S_1d]=first_drain(m_r,ph,pr,pl,x0_cas_end,n);
% Temperatures [1]:
T_low=30; % ideal fermentation temperature (C)
T_upper=39; % Scalding temperature of Norvegia (C)
D_T=T_upper-T_low; % heating difference (C)
% The heating of Norvegia to the scalding temperature takes 30-50 min
Heat_slope=D_T/30; % typical value 0.2-0.5 C/min
T_heat=95; % tempeperature of the hot water
% heating time:
heating_time=D_T/Heat_slope; % (min)
% intervall length:
L=1e+3;

% Temperature and time intervall:
T_int=linspace(T_low,T_upper,L); % C
time_heat_int=linspace(0,heating_time,L); % min

% Average heat capacities at normal temperatures (0-120 C) [2]:kJ kg^-1 C^-1
% molecular to mass heatcapacity 
CP_water=75/(16+2*1.008); % water
CP_protein=0.298*4.184; % [4] 
CP_lip=210/102.1; % Valeric acid (liquid)
CP_lac=418/342.3;
CP_butyric_acid=179/88.1; % butyric acid (liquid)
CP_salt=73/(40.08+2*35.45); 
% average heat capacity of the liquid:
CP_liq=C_w*CP_water+C_whey*CP_protein+C_lac*CP_lac+C_lact*CP_butyric_acid+C_bio*CP_water+C_min*CP_salt;
% average heat capacity of the solids i the coagel, assuming that the heat
% cappacity is equal the heatcapacity of casein:
CP_coagel=CP_protein*1/2+CP_lip*1/2;

% Heating dempands (kj/C):
m_whey_in=m_vat(end)-m_coagel; % kg whey solution
H_coa=m_coagel*CP_coagel; % solids
H_liq=m_whey_in*CP_liq; % whey solution 

% The sum of all added water is the amount of water needed to reach the
% final temperature, by assuming an isoentropic
% system:m_c*CP_c*dT_c=m_h*CP_h*dT_h
m_water_end=(H_coa+H_liq)*(T_upper-T_low)/(CP_water*(T_heat-T_upper));
% Water added and whey solution interval:
m_water=linspace(0,m_water_end,L); % kg
V_water=m_water; % assuming density equal 1 kg/l
m_CaCl2=10/100*m_r/1000; % kg CaCl2
m_whey=m_whey_in+m_water;
% Calculating the whey solution composition over the heating, kg i/kg whey:
c_lac_h=(S_1d*V_int(end)/1000)./m_whey;
c_lactate_h=(P_1d*V_int(end)/1000)./m_whey;
c_w_h=(C_w*m_whey_in+m_water)./m_whey;
c_whey_h=(C_whey*m_whey_in)./m_whey;
c_bio_h=(B_1d*V_int(end)/1000)./m_whey;
c_min_h=(C_min*m_whey_in)./m_whey;
c_CaCl2_h=m_CaCl2./m_whey;
% Scalding time and temperature:
t_scalding=15; % 15-45 min for Norvegia
t_scalding_int=linspace(heating_time,heating_time+t_scalding,L); % min
T_scalding_int=ones(L,1)'.*T_upper; % C

% Scalding concentrations of the whey solution, assuming no change in composition:
c_lac_s=c_lac_h(end).*ones(L,1)'; % kg/kg
c_lactate_s=c_lactate_h(end).*ones(L,1)';
c_w_s=c_w_h(end).*ones(L,1)';
c_whey_s=c_whey_h(end).*ones(L,1)'; % whey protein
c_bio_s=c_bio_h(end).*ones(L,1)';
c_min_s=c_min_h(end).*ones(L,1)';
c_CaCl2_s=c_CaCl2_h.*ones(L,1)';
% total inervalls:
time_int=[time_heat_int t_scalding_int];
c_lac=[c_lac_h c_lac_s]; % kg/kg
c_lactate=[c_lactate_h c_lactate_s];
c_w=[c_w_h c_w_s];
c_whey=[c_whey_h c_whey_s];
c_bio=[c_bio_h c_bio_s];
c_min=[c_min_h c_min_s];
c_CaCl2=[c_CaCl2_h c_CaCl2_s];
% pH of the heating:
Ka=10^(-3.86); % acid constant of lactic acid (C3H6O3)
Mw= 90.08/1000; % kg/mol

F=0; % fat content of the whey, w%
SNF=(1-c_w)*100-F;% solids non-fat, w%
% density of the whey solution [3]:
rho_liq=100./(F/0.93+SNF/1.608+c_w.*100); % kg/l

% molar concentration of lactic acid (mol/l):
p=c_lactate./Mw.*rho_liq;

% Buffering capacity of milk:
pka_buffer=5.5;
pH_milk=6.7;
% Using the Henderson-Hasselbach equation to determine the needed molar
% ratio of base and acid: A_HA_ratio=[A^-]/[HA]
A_HA_ratio_0=10^(pH_milk-pka_buffer);
V_initial=V_int(end)+V_water;
V_end=V_initial(end)*ones(1,(length(p)-length(c_lac_h)));
V_Heat=[V_initial V_end];
C0_cas=m_coagel./(V_int(end)+V_Heat).*1e+3; % g/l
% Concentration of buffer: casein used as a measure to regulate [7]
alfa_s1=38./97.*C0_cas;% g/l
Mw_s1=(22+23.7)/2*1e+3; % g/mol (Da)
c_s1=alfa_s1./Mw_s1; % mol/l
alfa_s2=10./97.*C0_cas;
Mw_s2=25e+3;
c_s2=alfa_s2./Mw_s2;
kappa=36./97.*C0_cas;
Mw_kappa=19e+3;
c_kappa=kappa./Mw_kappa;
beta=13./97.*C0_cas;
Mw_beta=24e+3;
c_beta=beta./Mw_beta;
citric=0.2./100.*1e+3; % g/l
Mw_citric=192.124;
c_citric=citric./Mw_citric;

c_buff=8.*(c_s1+c_s2+c_kappa+c_beta+c_citric); % mol/l
HA_mol_0=c_buff.*1./(1+A_HA_ratio_0);
A_mol_0=c_buff-HA_mol_0;

% Actual buffer concentations:
HA_mol=HA_mol_0+p;
A_mol=A_mol_0-p;
pH_heat=pka_buffer+log10(A_mol./HA_mol);


figure(6)
plot(time_int,c_lac*100)
hold on
plot(time_int,c_lactate*100)
hold on
plot(time_int,c_whey*100)
hold on
plot(time_int,c_bio*100)
hold on
plot(time_int,c_min*100)
hold on
plot(time_int,c_CaCl2*100)
hold off
title('Composition of the disolved solids in the whey during the heating')
xlabel('Time, min')
ylabel('Composition, w/w %')
legend('lactose','lactate','whey','biomass', 'minerals','CaCl2')
end
