function ripening(m_r,ph,pr,pl,x0_cas_end,n,R2)
% By: Tomas Tungen Kjellstrm 
% Start date: 28.04.2017
% NTNU
% Details:
% This function models the ripening phase of cheese production. The
% ripening is devided into: internal and externel water and salt transport,
% proteolysis, lipolysis and metabolism of residual lactose and catabolism
% of citrate and lactate.

% Export details:

if nargin==0
    m_r=100;     % mass flow of raw milk
    ph=5;        % pressure on the inlet of the UF-membrane
    pr=4;        % retentate pressure
    pl=1;        % permeate pressure
    x0_cas_end=4;   % retentate (main stream) casein mass fraction, w/w %
    n=0.3;       % volume removal ratio of the first drainage
    R2=3.5;      % whey to curd mass ratio after second drainage
end

% Assumptions:
% uniformal dispersion of all component, exept for salt which enter the
% ripening with the salt dispersion equal the salt dispersion at the end of
% the salt bath.

% References:
% [1] Eskin, M., Shahindi, F. (2013). 
% Biochemistry of Foods.Third edition. San Diego: Academic Press

% [2] Zorrilla, S. Rubiolo, A. (1997).
% Kinetics of Casein Degradation during Ripening
% of Fynbo Cheese Salted with NaCl/KCl Brine. Volume 62. Pages:386-389.
% Journal of Food Science

% [3] Simal et al. (2001). Water and salt difusion during cheese ripening:
% efect of the external and internal resistances to mass transfer. vol 48.
% pages 269-275. Journal of Food Engineering

% [4] Y. H. Hui (2007). Handbook of Food Products Manufacturing. volume 2.
% John Wiley & Sons

% [5] DherbeCourt et al. (2010). Time Course and Specificity of Lipolysis
% in Swiss Cheese. Vol 58(22). pages 11732-11739. Journal of agricultural 
% and food chemistry

% Importing composition data from the draining to use as initial amounts of
% the components other than salt:
[m_curd,m_whey_in_int,c_coagel_con_int,c_water_con_int,c_lac_con_int,c_lactate_con_int,c_bio_con_int,c_whey_con_int,c_min_con_int,c_CaCl2_con_int,c_lip_con_int]=draining_continuous(m_r,ph,pr,pl,x0_cas_end,n,R2);

% importing values on the salt amounts:
[ratio_end,Time_salt,C_salt_av_tot,C_salt_tot,m_salt_end,x]=salting(m_r,ph,pr,pl,x0_cas_end,n,R2);

% Initial average amounts: kg
m_lipids_0=c_lip_con_int(end)*m_curd(end);
m_casein_0=c_coagel_con_int(end)*m_curd(end);
m_water_0=c_water_con_int(end)*m_curd(end);
m_lactose_0=c_lac_con_int(end)*m_curd(end);
m_lactate_0=c_lactate_con_int(end)*m_curd(end);
m_biomass_0=c_bio_con_int(end)*m_curd(end);
m_whey_0=c_whey_con_int(end)*m_curd(end);
m_salt_0=m_salt_end;
m_tot_0=m_lipids_0+m_casein_0+m_water_0+m_lactose_0+m_lactate_0+m_biomass_0+m_whey_0+m_salt_0;

% Proteolysis:[2]
 % Fanbo cheese, 1 order model
% Casein compositions: kg/kg total
alfa_s1_0=0.38*m_casein_0/m_tot_0;
alfa_s2_0=0.1*m_casein_0/m_tot_0;
kappa_0=0.36*m_casein_0/m_tot_0;
beta_0=0.13*m_casein_0/m_tot_0;
sigma_0=0.03*m_casein_0/m_tot_0;

% The proteolysis of alfa s1 follows 1 order kinetics, which is space
% oriented. The first letter represents if the slice was from the middle 
% (m) or the rim (r) of the cheese. The second letter represents if the value is
% designated to the core (c) or the concentric region of the slice (o): [2]
k_m_c=0.976e-2; % middle slice core sample 
k_m_o=0.992e-2; % [day^-1]
k_r_c=0.963e-2;% 
k_r_o=0.990e-2; % outer rim slice outer diameter of the sample 

% time intervall:
t=linspace(0,(31+30+31),1e+4); % three months
% Assuming only the alfa s1 to be proteolysed: F(t)/F0
alfa_s1_m_c=exp(-k_m_c.*t);
alfa_s1_m_o=exp(-k_m_o.*t);
alfa_s1_r_c=exp(-k_r_c.*t);
alfa_s1_r_o=exp(-k_r_o.*t);
alfa_s1_mean=(alfa_s1_m_c+alfa_s1_m_o+alfa_s1_r_c+alfa_s1_r_o)./4;


% Lipolysis:
% The lipolysis is done enzymatic either by intracellular enzymes (lyzis)
% of LAB bacterias or by extracelluar and intracellular release of
% esterases of propyonic bacterias [5]:
T=[1e-5 10 17 24 31 46]; % days after start of ripening
% level of degredation: mg FFA formed/g fat:
% without LAB only: etc. Norvegia, Gouda and Cheddar
FFA_LAB=[2083 2111 2158 2263 2422 2427].*10^(-3); 
% with propionic bacteria: etc. Jarlsberg and Swiss cheese
FFA_prop=[2046 2273 3601 9462 11361 12883].*10^(-3);

% Regression models of the FFA to fat ratio over time:
% 2 order polynomial: y(x)= p1*x^2+p2*x+p3
[FFA_LAB_poly, GOF_LAB]=fit(T',FFA_LAB','poly2');  
% 2 order exponential:y(x)=a*exp(b*x)+c*exp(d*x)
[FFA_prop_exp2, GOF_propionic]=fit(T',FFA_prop','exp2');  

% FFA concentrations: mg/kg total
% empirical:
C_FFA_LAB=FFA_LAB.*m_lipids_0./m_tot_0.*1000;
C_FFA_prop=FFA_prop.*m_lipids_0./m_tot_0.*1000;

% Analytic regression models:
C_FFA_LAB_poly = (FFA_LAB_poly(t).*m_lipids_0)./m_tot_0.*1000;
C_FFA_prop_exp2= (FFA_prop_exp2(t)).*m_lipids_0./m_tot_0.*1000;

figure(11)
subplot(2,1,1)
plot(t,alfa_s1_m_c.*100)
hold on
plot(t,alfa_s1_m_o.*100)
hold on
plot(t,alfa_s1_r_c.*100)
hold on
plot(t,alfa_s1_r_o.*100)
hold on
plot(t,alfa_s1_mean.*100)
title('Proteolysis of alfa s1 casein')
hold off
legend('middle slice core alfa s1','middele slice outer rim alfa s1','rim slice core alfa s1','rim slice rim diameter alfa s1','alfa s1 mean')
xlabel('time [days]')
ylabel('[F/F0 *100 (%)]')
subplot(2,1,2)
plot(t,C_FFA_LAB_poly)
hold on
scatter(T,C_FFA_LAB)
hold on
scatter(T,C_FFA_prop)
hold on
plot(t,C_FFA_prop_exp2)
title('Lipolysis during the ripeing')
hold off
xlabel('time [days]')
ylabel('[mg/kg]')
legend('FFA LAB poly','FFA LAB empirical','FFA prop empirocal','exp2')
end