function [ratio_end,Time_salt,C_salt_av_tot,C_salt_tot,m_salt_end,x]=salting(m_r,ph,pr,pl,x0_cas_end,n,R2)
% By: Tomas Tungen Kjellstrm 
% Start date: 12.04.2017
% NTNU
% Details:
% This function makes a model for the brining of the unmatured cheese and
% generates the needed brineing time.

% Export details:
% ratio_end is the salt to solid mass ratio at the end of the salting 
% (g salt/kg solids)
% Time_salt is the needed salting time (h)
% C_salt_av_tot is the average salt concentration in the cheese at a 
% given time between 0 and Time_salt 
% C_salt_tot is the mass per thousanth salt (g salt/ kg total) at given distances from 
% the surface(x) in the cheeese troughout the salting phase.
% m_salt_end is the total amount of salt in the cheese (g).
% x is the length intervall of the cheese for unidirectional diffusion (cm)

% Assumptions:
% The salt solute concentration is assumed constant, as a result of a 
% sufficiently volumetic ratio between the brine and the cheese.
% The only salt present in the solute is NaCl.
% Constant temperature and pressure of 15 C and 1 atm
% The effective diffusivity is the same for all low molecular ions (Na, Cl, Ca).
% There is no diffusion of components other than salt.

% Refrences:
% [1]
% Aylward, G., Findlay, T. (2008). SI Chemical Data. 
% Sixth edition. Milton, Australia: John Wiley & Sons Australia Ltd 
% [2]
% Floury, J et al. (2009). Reducing salt level in food: Part 2.
% Modelling salt diffusion in model cheese systems with regards to 
% their composition. Volume 4. Pages 1621-1628. Food Science and Technology
% [3]
% Hagenes, K. (2010). Produksjon av meieriprodukter. First edition.
% Oslo: Baneforlaget
if nargin==0
    m_r=100;     % mass flow of raw milk
    ph=5;        % pressure on the inlet of the UF-membrane
    pr=4;        % retentate pressure
    pl=1;        % permeate pressure
    x0_cas_end=4;   % retentate (main stream) casein mass fraction, w/w %
    n=0.3;       % volume removal ratio of the first drainage
    R2=3.5;      % whey to curd mass ratio after second drainage
end

% Importing data on the drained curd:
[m_curd,m_whey_in_int,c_coagel_con_int,c_water_con_int,c_lac_con_int,c_lactate_con_int,c_bio_con_int,c_whey_con_int,c_min_con_int,c_CaCl2_con_int,c_lip_con_int]=draining_continuous(m_r,ph,pr,pl,x0_cas_end,n,R2);

% Using fick`s law of unidirectional mass transfer:
% initial conditions [2]:
m_NaCl_0= m_curd(end)*c_min_con_int(end)*1000; % g NaCl
m_CaCl2_0=m_curd(end)*c_CaCl2_con_int(end)*1000; % g CaCl2
m0_s=(m_NaCl_0+m_CaCl2_0)/1000; % kg
Salt_0=m0_s/m_curd(end)*100; % % salt (g/100 g)

Mw_Na=22.99; % g/ mol
Mw_Ca=40.08;
Mw_Cl=35.45;
Mw_CaCl2=Mw_Ca+2*Mw_Cl;
Mw_NaCl=Mw_Na+Mw_Cl;

% Assuming all salt is solved in the solute as ions:
n_CaCl2_0=m_CaCl2_0/Mw_CaCl2; % mol CaCl2
n_NaCl_0=m_NaCl_0/Mw_NaCl; 

C_cl_0=(n_CaCl2_0*2+n_NaCl_0)/m_curd(end); % mol Cl^-/kg
C_Na_0=n_NaCl_0/m_curd(end); % mol/kg
C_Ca_0=n_CaCl2_0/m_curd(end); % mol/kg
% Boundary conditions are the ions concentration in the solute: 
% The usual range of salt concentration is 20-22 w%
C_s_tot=0.21; % kg salt /kg total
C_s_Cl=C_s_tot/Mw_NaCl*1000; % mol Cl^-/kg 
C_s_Na=C_s_Cl;
C_s_Ca=0; % the solute contain only NaCl

% Effective diffusivion constant: [3]
D_dmH=3.02e-10;     % dry matter 44.0 %, 1.5 % salt
D_dmL=3.43e-10;     % dry matter 37.0 %, 1.5 % salt
D_dmL_S_l=2.81e-10; % dry matter 37.0 %, 0.5 % salt

% Initial salt concentration depencency:
% Salt concentration difference
diffs=D_dmL-D_dmL_S_l;
% one point formula gives:a=dY/dX=(D2-D1)/(x2-x1)
a_s=(diffs)/(1.5-0.5); % >0
D_s=D_dmL_S_l+a_s*(Salt_0-0.5);
diff_a=D_dmL-D_s;
R_salt=D_s/D_dmL;% D_dmL has the identical salt concentration as D_dmH

% Dry matter dependency:
DM=100*(1-c_water_con_int(end));
diff_dm=D_dmH-D_dmL;
a_dm=diff_dm/(44-37); % <0
D_dm=D_dmH+a_dm*(DM-44);
R_dm=D_dm/D_dmH; % if DM>44 => R<0

% The effective diffusion coefficient, calculated relative to a known state
% with 44 % drymatter and 1.5 % initial salt concentration.
D=(D_dmH)*R_salt*R_dm *60^2*100^2% cm^2/h

% cheese dimensions:
% assuming quadric dimmension
S=1.5e+3;
x=linspace(1e-8,20,S); % cm
t_total=linspace(1e-8,24*1.5,S); % h
[X,T]=meshgrid(x,t_total);

% Cloride ions:
% molecular concentration at a given distance the surface at a given time:
C_Cl_mol=C_s_Cl+(C_cl_0-C_s_Cl).*erf(X./(2.*sqrt(D.*T))); % mol Cl^-/kg
% mass per thousandth of Cl^- in the curd at a place x and a time t:
C_Cl=C_Cl_mol.*Mw_Cl; % g/kg

% Natrium ions:
% assuming the effective diffusion constant of natrium to be equivalent of
% cloride.
% molecular concentration:
C_Na_mol=C_s_Na+(C_Na_0-C_s_Na).*erf(X./(2*sqrt(D.*T))); % mol Na^+/kg
% Mass per thousandth of Na^+ in the curd at a place x and a time t:
C_Na=C_Na_mol.*Mw_Na; % g/kg

% Calsium ions: reversive diffusion
C_Ca_mol=C_s_Ca+(C_Ca_0-C_s_Ca).*erf(X./(2.*sqrt(D.*T)));
% Mass per thousandth
C_Ca=C_Ca_mol.*Mw_Ca;

% Total salt concentration:
C_salt_tot=C_Cl+C_Na+C_Ca; % g salt/kg

% Creating an vector of the average salt consentration over time:
k=1;
for k=1:(S)
C_salt_av_tot(k)=mean(C_salt_tot(k,:))';
k=k+1;
end
% Average salt concentration in the cheese time-intervall: g/kg
C_salt_av_tot=C_salt_av_tot(1:end,:)';

% Change in salt consentration: Cs(t)=m_s(t)/m_tot(t)=(m0_s+ds)/(m0_tot+ds)
ds=(C_salt_av_tot./1000.*m_curd(end)-m0_s)./(1-C_salt_av_tot./1000); % kg

% Calculating the needed brining time at defined brine salinity:
% In a matured Norvegia the solid content is 58 w % and has a salt content 
% of 1.3 w %: [3]
C_solid_norvegia=0.58; % kg/kg
C_salt_norvegia=0.013*1000; % g/kg

% desired salt content ratio: g salt/kg dry matter
R_stop=C_salt_norvegia/C_solid_norvegia;

% Solid and water content during the brining: kg/kg
C_solid=((1-c_water_con_int(end))*m_curd(end)+ds)./(m_curd(end)+ds);
c_water=1-C_solid;

% Salt to solid content ratio: g salt/kg solid
R_salt=C_salt_av_tot./C_solid;

% Exponential regression vector of the salt to solid ratio over time:
% R_salt(t)=a*exp(b*t)+c*exp(d*t)
[R_salt_poly, gof3] = fit(t_total',R_salt,'poly3','Normalize','off')


% Iteration to reach the needed salt to solid mass ratio
k2=1; % iteration 1
err=1e-2; % allowed deviation
maxiter=1e+4; % maximum number of iterations
i=1; % initial guess, h
for k2=1:maxiter
 R_salt_poly(i);
 diff=R_salt_poly(i)-R_stop;
if k2>=maxiter
     fprintf('Maximum number of iterations reached, incorrect brine time obtained, diff= %f',diff(end));
     break
elseif abs(diff)<=err % Correct average concentration obtained
    break
end
 if diff<-err % salt deficit
  i_test=i+abs(diff)/10;
 elseif diff>err % salt surplus
     i_test=i-abs(diff)/10;
 end
if i_test<0
    i_test=0.5;
end
 i=i_test;
 k2=k2+1;
end

% gathering some relevant end-values:
ratio_end=R_salt_poly(i(end));
Time_salt =i(end) % h


C_Cl_mol_end=C_s_Cl+(C_cl_0-C_s_Cl).*erf(X./(2.*sqrt(D.*i(end)))); % mol Cl^-/kg
% mass per thousandth of Cl^- in the curd at a place x and a time t:
C_Cl_end=C_Cl_mol_end.*Mw_Cl; % g/kg
Cl_mean_end=mean(C_Cl_end);
C_Na_mol_end=C_s_Na+(C_Na_0-C_s_Na).*erf(X./(2*sqrt(D.*i(end)))); % mol Na^+/kg
% Mass per thousandth of Na^+ in the curd at a place x and a time t:
C_Na_end=C_Na_mol_end.*Mw_Na; % g/kg
Na_mean_end=mean(C_Na_end);
C_Ca_mol_end=C_s_Ca+(C_Ca_0-C_s_Ca).*erf(X./(2.*sqrt(D.*i(end))));
% Mass per thousandth
C_Ca_end=C_Ca_mol_end.*Mw_Ca;
Ca_mean_end=mean(C_Ca_end);
C_salt_mean_end=mean(Cl_mean_end+Na_mean_end+Ca_mean_end) % g/kg
ds_mean_end=(C_salt_mean_end/1000*m_curd(end)-m0_s)/(1-C_salt_mean_end/1000);
% Assuming all other components are homogenious dispersed troughout the
% cheese
C_solid_mean_end=((1-c_water_con_int(end))*m_curd(end)+ds_mean_end)/(m_curd(end)+ds_mean_end);
C_water_mean_end=1-C_solid_mean_end;
m_tot_end=m_curd(end)+ds(end);
m_salt_end=C_salt_mean_end.*m_tot_end; % g salt

% FJERNE????
% alternative time intervall:
t=linspace(0,24*6,13);

% error function responces at a given time
R_0=erf(x./(2.*sqrt(D.*t(1)))); % hg/kg
R_12=erf(x./(2.*sqrt(D.*t(2)))); % hg/kg
R_1_d=erf(x./(2.*sqrt(D.*t(3)))); % hg/kg
R_36=erf(x./(2.*sqrt(D.*t(4)))); % hg/kg
R_2_d=erf(x./(2.*sqrt(D.*t(5)))); % hg/kg
R_60=erf(x./(2.*sqrt(D.*t(6)))); % hg/kg
R_3_d=erf(x./(2.*sqrt(D.*t(7)))); % hg/kg
R_84=erf(x./(2.*sqrt(D.*t(8)))); % hg/kg
R_4_d=erf(x./(2.*sqrt(D.*t(9)))); % hg/kg
R_108=erf(x./(2.*sqrt(D.*t(10)))); % hg/kg
R_5_d=erf(x./(2.*sqrt(D.*t(11)))); % hg/kg
R_132=erf(x./(2.*sqrt(D.*t(12)))); % hg/kg
R_6_d=erf(x./(2.*sqrt(D.*t(13)))); % hg/kg
figure(7)
plot(x,R_0)
hold on
plot(x,R_1_d)
hold on
plot(x,R_2_d)
hold on
plot(x,R_3_d)
hold on
plot(x,R_4_d)
hold on
plot(x,R_5_d)
hold on
plot(x,R_6_d)
hold off
ylabel('ratio responce')
xlabel('distance from surface (cm)')
legend('initial','1 day','2 days', '3 days','4 days','5 days','6 days')
title('Errorfunction response of distance from surface at different times')
hold off

figure(8)
mesh(T,X,C_salt_tot,'FaceLighting','gouraud','LineWidth',0.3)
xlabel('time [h]')
ylabel('Distance from surface, cm')
zlabel('salt concentration [g/kg]')
title('Surface plot of the salt concentration over distance from surface and time')
hold off

figure(9)
title('Average salt concentration over brining time')
hold on
plot(T,C_salt_av_tot)
hold on
plot(Time_salt,C_salt_mean_end,'o')
xlabel('time [h]') 
ylabel('salt concentration [g/kg total]')
hold off

figure(10)
title('Average salt over solids ratio over time')
hold on
scatter(t_total,R_salt)
hold on
plot(R_salt_poly,'r')
hold off
xlabel('time [h]')
ylabel('g salt/kg solid')
end