function [m_vat_int,m_whey_int,c_coa_int,c_lac_int,c_lactate_int,c_w_int,c_bio_int,c_whey_int,c_min_int,c_lip_int,c_CaCl2_int,rho_liq]=second_drain(m_r,ph,pr,pl,x0_cas_end,n,R2)
% By: Tomas Tungen Kjellstrm 
% Start date: 07.04.2017
% NTNU
% Details:
% This function makes a model for the second drainage, after the coagel has
% heated and the microbial activity has reached its peak.

% Export details:
% m_vat_int is the total mass in the vat at a given time between the start and
% end of the second drain.
% m_whey_int is the mass of whey solution in the vat at a given time between 
% the start and end of the second drain.
% c_coa_int, c_lac_int, c_lactate_int, c_w_int, c_bio_int, c_whey_int, c_min_int
% c_lip_int and c_CaCl2_int are the total mass fractions of the coagel, lactose, lactate, 
% water, biomass, whey protein, NaCl, fat, CaCl2 in the vat at a given time during the second drainage.
% rho_liq is the whey density, kg/l

% Refrences:
% [1]
% Bylund, G. (2003). Dairy processing handbook.
% Second edition. Malm, Sweden: Tetra Pak Processing Systems AB

% [2]
% Daviau, C. (2000). Rennet coagulation of skim milk and curd drainage:
% Effect of pH, casein concentration, ionic strength 
% and heat treatment Vol 80. page
% 397-415. Le Lait 

if nargin==0
    m_r=100;     % mass flow of raw milk (kg)
    ph=5;        % feed pressure of the UF-membrane (kPa)
    pr=4;        % retentate pressure (kPa)
    pl=1;        % permeate pressure (kPa)
    x0_cas_end=4;   % retentate (main stream) casein mass fraction, w/w %
    n=0.3;       % volume removal ratio of the first drainage
    R2=3.5;      % whey to curd mass ratio at the end of second drainage
end

% assumptions:
% The draining is exponential decaying.
% No coagel is lost during the draining (ideal system)
% Constant temperature and pressure of 30 C and 1 atm.


% Importing numerics from the heating:
[pH_heat,c_lac,c_lactate,c_w,c_bio,c_whey,c_min,c_CaCl2,m_whey]= heating(m_r, ph, pr, pl,x0_cas_end,n);
% Importing amount of persipitate in the curd: 
[V_inter,m_vat,C_whey,C_lac,C_lactate,C_bio,C_min,C_w,m_coagel_solids,pH_1_d_end,B_1d_end,P_1d_end,S_1d_end]=first_drain(m_r,ph,pr,pl,x0_cas_end,n);
% Importing the lipid amount in the vat, assuming all lipid is conserved in
% the vat:
[R, x0_w,x0_lip,x0_lac,x0_min,x0_cas,x0_whey,x0_whey_de,x0_NPN]=UF_filter(m_r, ph, pr, pl,x0_cas_end);

% Calculating the density of the whey solution [1]:
% fat and solids non-fat mass persentage:
F=0; 
SNF=100*(c_lac(end)+c_lactate(end)+c_bio(end)+c_whey(end)+c_min(end)+c_CaCl2(end));
rho_liq=100/(F/0.93+SNF/1.608+c_w(end)*100); % kg/l

% Initial whey solution volume, l:
V_whey_0=m_whey(end)/rho_liq;
% Inital total composition:

m_tot_0=m_coagel_solids+m_whey(end); % initial mass in the vat

% Calculating the mass of whey left in the vat after the second drain:
% R2=m_whey/m_solids
m_whey_end = R2*m_coagel_solids;
% Volume:
V_whey_end=m_whey_end/rho_liq;
% Whey removed:
V_whey_rem=V_whey_0-V_whey_end;

% Drainage kinetics [2]: Mesophilic, CN=-1, IS=-1
W_1=893/2*0.95; % g/kg expelled (95 % expelled at time t_1)
t_1=53;    % min
W_2=827/2*0.95;
t_2=10.25;

% Calculating the needed draining time:
% W=(m0-m(t))/m_tot0*10^3
W=(m_whey(end)-m_whey_end)/m_tot_0*1e+3; % g/ kg
% At infinite time the numerator reaches zero, since W is a function of W_1
% and W_2:
t_d2=log(W_1+W_2-W)/((log(W_1)/t_1)+(log(W_2)/t_2)); % typically 0-7 min
S2=1e+2;
% time, draining and mass in the vat intervall:
t_d_int=linspace(0,t_d2,S2); % min
W_int=W_1*(1-exp(-t_d_int./t_1))+W_2*(1-exp(-t_d_int./t_2)); %g/kg initial
m_whey_int=m_whey(end)-W_int.*m_tot_0*1e-3;
m_vat_int=m_coagel_solids+m_whey_int;

% Total composition intervall of the seconod drain, mass fraction:
%m_vat_int=m_coagel_solids+V_whey_inter.*rho_liq; 
c_lip_int=(R*x0_lip)./m_vat_int; % kg/kg total
c_coa_int=m_coagel_solids./m_vat_int-c_lip_int; % kg casein/kg total
c_lac_int=c_lac(end).*m_whey_int./m_vat_int;
c_lactate_int=c_lactate(end).*m_whey_int./m_vat_int;
c_w_int=c_w(end).*m_whey_int./m_vat_int;
c_bio_int=c_bio(end).*m_whey_int./m_vat_int;
c_whey_int=c_whey(end).*m_whey_int./m_vat_int;
c_min_int=c_min(end).*m_whey_int./m_vat_int;
c_CaCl2_int=c_CaCl2(end).*m_whey_int./m_vat_int;
end